package sample.app1_dev_vraa;

import lv.gov.eps.ivis.iss.requestserviss.v1_0.ISServiceV10;
import lv.gov.eps.ivis.iss.requestserviss.v1_0.SubmitSync;
import lv.gov.eps.ivis.iss.requestserviss.v1_0.SubmitSyncResponse;
import lv.gov.eps.ivis.iss.requestserviss.v1_8.RequestWebService;
import lv.gov.eps.ivis.xmlschemas._100000.testisservise.v1_0.CalculationStructure;
import lv.gov.eps.ivis.xmlschemas._100000.testisservise.v1_0.ObjectFactory;
import lv.gov.eps.ivis.xmlschemas._100000.testisservise.v1_0.OperationType;
import lv.gov.eps.ivis.xmlschemas._100001.ivis.v1_0.IVISRequestStructure;
import lv.gov.eps.ivis.xmlschemas._100001.ivis.v1_0.RequestHeaderStructure;
import org.apache.cxf.Bus;
import org.apache.cxf.BusException;
import org.apache.cxf.BusFactory;
import org.apache.cxf.bus.spring.SpringBusFactory;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.endpoint.EndpointException;
import org.apache.cxf.message.Message;
import org.apache.cxf.ws.security.trust.STSClient;

import javax.xml.bind.JAXBElement;
import javax.xml.ws.BindingProvider;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class RequestSync_Certificate {

    public static void main(String[] args) throws EndpointException, BusException {

        //Spring context
        SpringBusFactory bf = new SpringBusFactory();
        Bus bus = bf.createBus("app1_dev_vraa/client-cert.xml");
        BusFactory.setDefaultBus(bus);

        //WS Service
        RequestWebService service = new RequestWebService();
        ISServiceV10 port = service.getWs2007FederationNoSct();

        // Service Configuration
        // Set STS and RequestService Parameters OnTheFly
        Map<String, Object> ctx = ((BindingProvider) port).getRequestContext();
        ctx.put(Message.ENDPOINT_ADDRESS, "https://app1-dev-vraa.abcsoftware.lv/Request.WebService/v1-9/WcfService/ws2007FederationNoSct");

        // STS Configuration
        STSClient stsClient = (STSClient) ctx.get("ws-security.sts.client");
        Client client = stsClient.getClient();
        client.getRequestContext().put(Message.ENDPOINT_ADDRESS, "https://app1-dev-vraa.abcsoftware.lv/Viss.PfasAuth.STS/Pfas.STS/stable/STS/Issue.svc/trust/13/certificatemixed");
        Map<String, Object> stsProps = new HashMap();
        stsClient.setProperties(stsProps);

        // Start transaction.
        String transaction = port.startTransaction("URN:IVIS:100001:EP-EP22-V1-3");
        assert transaction != null : "Tranzakcija nav uzsakta";

        // Request Header
        IVISRequestStructure requestStructure = new IVISRequestStructure();
        requestStructure.setHeader(new RequestHeaderStructure());
        requestStructure.getHeader().setMessageID(UUID.randomUUID().toString());
        requestStructure.getHeader().setMessageType("URN:IVIS:100001:XSD-Testing-TestISServise-v1-0-TYPE-Calculation");
        requestStructure.getHeader().setCorrelationID(UUID.randomUUID().toString());
        requestStructure.getHeader().setTransactionID(transaction);
        requestStructure.getHeader().setDestination("URN:IVIS:100001:ISS-SIA.ABC-CalculationServiceSync-v2-0");

        // Calculation Service
        CalculationStructure structure = new CalculationStructure();
        structure.setNumber1(new BigInteger("1"));
        structure.setNumber2(new BigInteger("2"));
        structure.setOperation(OperationType.ADDITION);
        ObjectFactory of = new ObjectFactory();
        JAXBElement<CalculationStructure> calculation = of.createCalculation(structure);
        IVISRequestStructure.Body body = new IVISRequestStructure.Body();
        body.setAny(calculation);
        requestStructure.setBody(body);

        // Submit Sync
        SubmitSync submitSync = new SubmitSync();
        submitSync.setIVISRequest(requestStructure);

        // Submit Response
        SubmitSyncResponse subR = port.submitSync(submitSync);
        assert subR.getIVISResponse().getHeader().getResult().equals("success");

        // End Transaction!
        port.endTransaction(transaction);
    }
}
